<?php
//
// Copyright (c) 2007 Satoshi Fukutomi <info@fuktommy.com>.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

class FeedGetter
{
    public $cacheDir = '/var/cache/feed';
    public $lockFile = '/var/cache/feed/lock';

    public function get($url, $expire = 3600)
    {
        $cacheName = md5($url);
        $cacheFile = $this->cacheDir . '/' . $cacheName;
        if (((! is_file($cacheFile)) || (intval(filemtime($cacheFile)) + $expire < time())) &&
            ($lock = fopen($this->lockFile, 'w')) && flock($lock, LOCK_EX | LOCK_NB)) {
            $buf = file_get_contents($url);
            flock($lock, LOCK_UN);
            if ($buf === false) {
                if (is_file($cacheFile)) {
                    return unserialize(file_get_contents($cacheFile));
                } else {
                    return array();
                }
            }
        } else {
            return unserialize(file_get_contents($cacheFile));
        }
        $xml = simplexml_load_string($buf);
        if ($xml === false) {
            return array();
        }
        $items = array();
        if (isset($xml->item[0])) {
            foreach ($xml->item as $item) {
                $items[] = array('title'        => strval($item->title),
                                 'link'         => strval($item->link),
                                 'description'  => strval($item->description));
            }
        } elseif (isset($xml->channel->item[0])) {
            foreach ($xml->channel->item as $item) {
                $items[] = array('title' => strval($item->title),
                                 'link'  => strval($item->link));
            }
        }
        if (count($items) > 0) {
            file_put_contents($cacheFile, serialize($items));
        } elseif (is_file($cacheFile)) {
            return unserialize(file_get_contents($cacheFile));
        }
        return $items;
    }
}

?>
